@@ -20,6 +20,7 @@
 That is import as expression, and star import.
 """
 
+from nuitka.nodes.ImportNodes import hard_modules
 from nuitka.nodes.LocalsScopes import GlobalsDictHandle
 from nuitka.PythonVersions import python_version
 
@@ -150,23 +151,21 @@ def generateImportModuleNameHardCode(to_name, expression, emit, context):
 
         if module_name == "sys":
             emit("""%s = PySys_GetObject((char *)"%s");""" % (value_name, import_name))
-        elif module_name in ("os", "__future__", "importlib._bootstrap"):
+        elif module_name in hard_modules:
             emitLineNumberUpdateCode(emit, context)
 
+            # TODO: The import name wouldn't have to be an object really, could do with a
+            # C string only.
             emit(
                 """\
 {
-    PyObject *hard_module = PyImport_ImportModule("%(module_name)s");
-    if (likely(hard_module != NULL)) {
-        %(to_name)s = PyObject_GetAttr(hard_module, %(import_name)s);
-    } else {
-        %(to_name)s = NULL;
-    }
+    PyObject *hard_module = IMPORT_HARD_%(module_name)s();
+    %(to_name)s = LOOKUP_ATTRIBUTE(hard_module, %(import_name)s);
 }
 """
                 % {
                     "to_name": value_name,
-                    "module_name": module_name,
+                    "module_name": module_name.upper(),
                     "import_name": context.getConstantCode(import_name),
                 }
             )
