@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -185,7 +185,7 @@ PyObject *CALL_FUNCTION_WITH_ARGS%(args_count)d(PyObject *called, PyObject **arg
                 return NULL;
             }
 #else
-            PyErr_Format(
+            SET_CURRENT_EXCEPTION_TYPE0_FORMAT1(
                 PyExc_TypeError,
                 "%%s() takes no arguments (%(args_count)d given)",
                 ((PyCFunctionObject *)called)->m_ml->ml_name
@@ -230,7 +230,7 @@ PyObject *CALL_FUNCTION_WITH_ARGS%(args_count)d(PyObject *called, PyObject **arg
                 return NULL;
             }
 #else
-            PyErr_Format(PyExc_TypeError,
+            SET_CURRENT_EXCEPTION_TYPE0_FORMAT1(PyExc_TypeError,
                 "%%s() takes exactly one argument (%(args_count)d given)",
                  ((PyCFunctionObject *)called)->m_ml->ml_name
             );
@@ -252,7 +252,7 @@ PyObject *CALL_FUNCTION_WITH_ARGS%(args_count)d(PyObject *called, PyObject **arg
             }
 #endif
 
-#if PYTHON_VERSION < 360
+#if PYTHON_VERSION < 0x360
             if (flags & METH_KEYWORDS) {
                 result = (*(PyCFunctionWithKeywords)method)(self, pos_args, NULL);
             } else {
@@ -262,7 +262,7 @@ PyObject *CALL_FUNCTION_WITH_ARGS%(args_count)d(PyObject *called, PyObject **arg
             if (flags == (METH_VARARGS|METH_KEYWORDS)) {
                 result = (*(PyCFunctionWithKeywords)method)(self, pos_args, NULL);
             } else if (flags == METH_FASTCALL) {
-#if PYTHON_VERSION < 370
+#if PYTHON_VERSION < 0x370
                 result = (*(_PyCFunctionFast)method)(self, &PyTuple_GET_ITEM(pos_args, 0), %(args_count)d, NULL);
 #else
                 result = (*(_PyCFunctionFast)method)(self, &pos_args, %(args_count)d);
@@ -348,7 +348,7 @@ PyObject *CALL_METHOD_WITH_ARGS%(args_count)d(PyObject *source, PyObject *attr_n
         if (descr != NULL) {
             Py_INCREF(descr);
 
-#if PYTHON_VERSION < 300
+#if PYTHON_VERSION < 0x300
             if (PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)) {
 #endif
                 func = Py_TYPE(descr)->tp_descr_get;
@@ -364,7 +364,7 @@ PyObject *CALL_METHOD_WITH_ARGS%(args_count)d(PyObject *source, PyObject *attr_n
                     Py_DECREF(called_object);
                     return result;
                 }
-#if PYTHON_VERSION < 300
+#if PYTHON_VERSION < 0x300
             }
 #endif
         }
@@ -453,12 +453,12 @@ PyObject *CALL_METHOD_WITH_ARGS%(args_count)d(PyObject *source, PyObject *attr_n
             return result;
         }
 
-#if PYTHON_VERSION < 300
-        PyErr_Format(
+#if PYTHON_VERSION < 0x300
+        SET_CURRENT_EXCEPTION_TYPE0_FORMAT2(
             PyExc_AttributeError,
             "'%%s' object has no attribute '%%s'",
             type->tp_name,
-            PyString_AS_STRING( attr_name )
+            PyString_AS_STRING(attr_name)
         );
 #else
         PyErr_Format(
@@ -470,7 +470,7 @@ PyObject *CALL_METHOD_WITH_ARGS%(args_count)d(PyObject *source, PyObject *attr_n
 #endif
         return NULL;
     }
-#if PYTHON_VERSION < 300
+#if PYTHON_VERSION < 0x300
     else if (type == &PyInstance_Type) {
         PyInstanceObject *source_instance = (PyInstanceObject *)source;
 
@@ -528,7 +528,7 @@ PyObject *CALL_METHOD_WITH_ARGS%(args_count)d(PyObject *source, PyObject *attr_n
             }
 
         } else if (unlikely(source_instance->in_class->cl_getattr == NULL)) {
-            PyErr_Format(
+            SET_CURRENT_EXCEPTION_TYPE0_FORMAT2(
                 PyExc_AttributeError,
                 "%%s instance has no attribute '%%s'",
                 PyString_AS_STRING(source_instance->in_class->cl_name),
@@ -596,7 +596,7 @@ PyObject *CALL_METHOD_WITH_ARGS%(args_count)d(PyObject *source, PyObject *attr_n
         Py_DECREF(called_object);
         return result;
     } else {
-        PyErr_Format(
+        SET_CURRENT_EXCEPTION_TYPE0_FORMAT2(
             PyExc_AttributeError,
             "'%%s' object has no attribute '%%s'",
             type->tp_name,
