@@ -320,7 +320,13 @@ ImportScanFinding = collections.namedtuple(
 unworthy_namespaces = ("Cython", "lxml")
 
 
-def _reportCandidates(module_name, candidate, candidates):
+def _reportCandidates(package_name, module_name, candidate, candidates):
+    module_name = (
+        package_name.getChildNamed(module_name)
+        if package_name is not None
+        else module_name
+    )
+
     if (
         candidate.priority == 1
         and Options.shallPreferSourcecodeOverExtensionModules() is None
@@ -341,7 +347,7 @@ existing '%s' extension module by default. Candidates were: %s <-> %s."""
                     )
 
 
-def _findModuleInPath2(module_name, search_path):
+def _findModuleInPath2(package_name, module_name, search_path):
     """This is out own module finding low level implementation.
 
     Just the full module name and search path are given. This is then
@@ -436,13 +442,23 @@ def _findModuleInPath2(module_name, search_path):
 
         # On case sensitive systems, no resolution needed.
         if case_sensitive:
-            _reportCandidates(module_name, candidates[0], candidates)
+            _reportCandidates(
+                package_name=package_name,
+                module_name=module_name,
+                candidate=candidates[0],
+                candidates=candidates,
+            )
             return candidates[0].full_path
         else:
             for candidate in candidates:
                 for fullname, _filename in listDir(candidate[0]):
                     if fullname == candidate.full_path:
-                        _reportCandidates(module_name, candidate, candidates)
+                        _reportCandidates(
+                            package_name=package_name,
+                            module_name=module_name,
+                            candidate=candidate,
+                            candidates=candidates,
+                        )
                         return candidate.full_path
 
             # Only exact case matches matter, all candidates were ignored,
@@ -549,7 +565,7 @@ def _findModuleInPath(module_name):
 
     try:
         module_filename = _findModuleInPath2(
-            module_name=module_name, search_path=search_path
+            package_name=package_name, module_name=module_name, search_path=search_path
         )
     except SyntaxError:
         # Warn user, as this is kind of unusual.
