@@ -745,7 +745,7 @@ class UncompiledPythonPackage(UncompiledPythonModule):
 class PythonMainModule(CompiledPythonModule):
     kind = "PYTHON_MAIN_MODULE"
 
-    __slots__ = ("main_added",)
+    __slots__ = ("main_added", "early_modules")
 
     def __init__(self, main_added, mode, future_spec, source_ref):
         # Is this one from a "__main__.py" file.
@@ -760,6 +760,8 @@ class PythonMainModule(CompiledPythonModule):
             source_ref=source_ref,
         )
 
+        self.early_modules = ()
+
     def getDetails(self):
         return {
             "filename": self.source_ref.getFilename(),
@@ -830,6 +832,20 @@ class PythonMainModule(CompiledPythonModule):
         else:
             return CompiledPythonModule.getOutputFilename(self)
 
+    def setEarlyModules(self, early_modules):
+        self.early_modules = early_modules
+
+    def getEarlyModules(self):
+        return self.early_modules
+
+    def computeModule(self):
+        CompiledPythonModule.computeModule(self)
+
+        from nuitka.ModuleRegistry import addUsedModule
+
+        for early_module in self.early_modules:
+            addUsedModule(early_module)
+
 
 class PythonShlibModule(PythonModuleBase):
     kind = "PYTHON_SHLIB_MODULE"
